% demo_PHASE_LA.

% Demo examples to the paper 
% "Phase local approximation (PhaseLa) technique for phase unwrap from
% noisy data" ,Vladimir Katkovnik, Jaakko Astola, Karen Egiazarian 

% Submitted to IEEE Trans on Image Processing, May 2007

% Phase unwrapping from noisy data 
% Input data is noisy wrapped phase

% Pointwise linear LPA with the ICI adaptive window sizes is used for the pixel wise filtering;
% Unwrapping is enabled by the phase tracking.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% We wish specially to note that:
% PROGRAMS: zpim.m, zs.m, zs.dill, zstep.m, used in this demo for comparison of the results,
% implement the ZSTEP and ZpiM  algorithms from the paper
% J. Dias and J. Leitao, "The ZpiM algorithm: a method for interferometric 
% image  reconstruction in SAR/SAS," IEEE Trans Image Processing, vol. 11, n? 4, pp. 408--422, 2002.
% These programs are available on WEBSITE: 
% Jose Manuel Bioucas Dias              http://www.lx.it.pt/~bioucas/
% Instituto de Telecomunicacoes         http://red.lx.it.pt/~mtf/TCRP/
% Instituto Superior Tecnico
% The auxiliary programs gauselle.m, insairpair.r, dbuild.m are also taken
% from this website 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

% V. Katkovnik, Tampere University of Technology, Tampere, Finland, October 2007

close all
clear all
tic
'-------------------'
'START demo_PHASE_LA'
'-------------------'
global y_true z_est

%%%%%%%%%%%% Main Parameters of the algorithm %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sigma=.5; % [0, .2 .3 .4 .5 .75 ]% SIGMA FOR ADDITIVE NOISE IN OBSERVATIONS
coherence=.8; %% Coherence for 'J.M.B.Dias and J.M.N.Leitao models'
SNR=0; % Signal-to-noise ration used only for model 15
global HH
%%%%%%% LPA window size %%%%%%%%%%%%%%%%%%%%%%
%% HH is a set of the window sizes used for estimation and window size adaptation
%  HH=[2 3 4 5];       % recommended for test=14, 'J.M.B.Dias and J.M.N.Leitao models, sigma=0'
  
HH=[1 2 3 4]; 
% HH=[3 5 7 9]; % For ramp test-function, test=3

true_initialization=1;  % true_initialization=1 gives the accurate initialization for the first pixel
% true_initialization=0 gives the initialization by the estimates from
% observations
gamma=2; % THE ICI parameter, recommended value
% gamma=5; % For ramp test-function, test=3

test=15; % Number of the model selected for experiment 

if test==14
    HH=[2 3 4 5];
  
   gamma=2.;
   gamma=1.2;
   sigma=.0;
   % amplitude of the signal can be installed in function_TruePhaseModel.m
    
end
if test==15
    SNR=8;
end

iteration_number=20; % Maximum number of iterations in Gauss-Newton recursive estimation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% TRUE PHASE MODELS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


[y_true, z_est]=function_TruePhaseModel(test, coherence, sigma,SNR);


mask=ones(size(y_true));  % mask=1 indicate the image parts used for estimation
                           % mask=0 parts are not used for estimation.


% test=1   'Quadratic Model'
% test=2   'Sinc Model '
% test=3   'Linear (ramp) Model'
% test=4   'Mexican Hat Model'
% test=6   'Gaussian Model'
% test=7   'Cosine Model'
% test=11  'Pyramid'
% test=14  'J.M.B.Dias and J.M.N.Leitao models' % 
% test=15  'Le Ying, at al., models' % 
ddelta=1;
[y_x, y_y]=function_Differentiation(y_true,ddelta); % True phase derivatives


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% NOISY OBSERVATION DATA%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

S=2055615866; randn('seed', S);
 
 %%%%%%%%%%%%%%%%%%%% NOISY PHASE MODEL  %%%%%%%%%%%%%%%%%%
   
     if (test~=14)&(test~=15)
  
 B=ones(size(y_true));
 z_x11=B.*cos(y_true)+sigma*randn(size(y_true)); %%% NOISE IN SINE/COSINE
 z_x22=B.*sin(y_true)+sigma*randn(size(y_true));
 z_est=angle(z_x11+j*z_x22);
 z_est_true=wrap(y_true+sigma*randn(size(y_true)));
     end
 
    

mask=ones(size(y_true));  % mask=1 indicate the image parts used for estimation
   
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     
  z_x1=cos(z_est); %%% COS observations (for the algorithm)
  z_x2=sin(z_est); %%% SIN observations  (for the algorithm)
  
 % noisy phase derivatives 
 
 [z_x_est, z_y_est]=function_Differentiation(medfilt2(z_est,[5,5]),ddelta);
 z_x_est=(z_x_est)/ddelta; z_y_est=(z_y_est)/ddelta;

 
 %%%%%%%%%% END OF NOISY OBSERVATION MODEL %%%%%%%%%%%%%%%%%%%%%%%
 
 
 
 
 
 %%%%%%%%%%%%% LPA_ICI_unwrap is the main procedure %%%%%%%%%%%%%%
 %% It returns:
 %% The estimate of the phase phi_estimate, and the derivatives phi_x_estimate, phi_y_estimate
 %% h_opt adaptive window sizes

     
      

 [phi_estimate, phi_x_estimate,phi_y_estimate,h_opt,PHI_estimate_h]=...
     LPA_ICI_Unwrap(y_true,y_x,y_y,z_x1,z_x2,z_est,z_x_est,z_y_est,HH,true_initialization,mask,gamma);
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%         
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
 
 %%%%%%%%%%%%% Illustrations and results %%%%%%%%%%%%%%%%%%%%%
 
 if length(HH)>1
 figure, imshow(h_opt,[]), colorbar, title('ICI adaptive windows')
 end
 
 phi_estimate_zstep=zstep(z_est); % Noisy Absolute Phase Recosntructed 
                         % by z-step algorithm
 
if test==15
    y_true=y_true/2/pi; phi_estimate=phi_estimate/2/pi;
    phi_estimate_zstep=phi_estimate_zstep/2/pi;
end

if 1>0  % Estimates Obtained by zpim algorithm 
    %  from J. Dias and J. Leitao, "The ZpiM algorithm: a method for interferometric 
    %  image reconstruction in SAR/SAS," IEEE Trans Image Processing, vol. 11, n? 4, pp. 408--422, 2002.
 apha=zpim(z_est);
 if test==15
    apha=apha/2/pi; 
end
 sds_ICI_unwrap1=function_Errors_NoNormalization(y_true,apha,phi_estimate_zstep ,2);
 figure, surfl(apha);shading interp;colormap(gray);
 title(['Z\pim, RMSE = ', num2str(sds_ICI_unwrap1(5)) ])

 end


figure, surfl(y_true);shading interp;colormap(gray);
title('Absolute Phase');

figure; surfl(phi_estimate_zstep);shading interp; colormap(gray);
title('Noisy Absolute Phase');




%%%%%%%% FIXED h ESTIMATES from Phasela algorithm %%%%%%%%%%%%%%%%%%%
if length(HH)>1
for ss_h=1:length(HH)
    if test==15
    PHI_estimate_h(:,:,ss_h)=PHI_estimate_h(:,:,ss_h)/2/pi;
    end

    
    display (['h=', num2str(HH(ss_h))]);     
    sds_ICI_unwrap11=function_Errors_NoNormalization(y_true,PHI_estimate_h(:,:,ss_h),phi_estimate_zstep ,2);
    
    figure, surfl(PHI_estimate_h(:,:,ss_h));shading interp;colormap(gray);
    title(['h=', num2str(HH(ss_h)), ', RMSE = ', num2str(sds_ICI_unwrap11(5)) ])

end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% PhaseLa Estimate %%%%%%%%%%%%%%%%%%%
display ('PhaseLa Estimate')

 sds_ICI_unwrap=function_Errors_NoNormalization((y_true),(phi_estimate),phi_estimate_zstep,2);


 figure, surfl(phi_estimate);shading interp;colormap(gray);
 title(['PhaseLa, RMSE = ', num2str(sds_ICI_unwrap(5)) ])
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


 display ('PhaseLa Time')
 
 
toc
%%%%%%%%%%%% Comparison %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %%%%%%%% ZpiM algorithm %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 if 0 %(test==14)|(test==15)
     disp('Z\pi M algorithm')
tic 
apha=zpim(z_est);
toc
'ZpiM'
if test==15
    apha=apha/2/pi;
end

sds_zpim2=function_Errors_NoNormalization(y_true, apha, phi_estimate_zstep,2);

figure, surfl(apha);shading interp;colormap(gray);
title(['Z\piM, RMSE = ', num2str(sds_zpim2(5)) ])

 end

 if 1<0
     tic
 display ('Z\pim')


 apha=zpim(z_est);
 sds_ICI_unwrap1=function_Errors_NoNormalization(y_true,apha,phi_estimate_zstep ,2);
 figure, surfl(apha);shading interp;colormap(gray);
 title(['Z\pim, RMSE = ', num2str(sds_ICI_unwrap1(5)) ])
 toc

 end
